import logging
import os
import random
import string
import imageio
import datetime


from multiprocessing import Pool
from PIL import Image

LOGGER = logging.getLogger("TIMELAPSE")

class TimelapseFactory:
    @staticmethod
    def factory(type):
        if type == "GIF":
            return GifTimeLapse()
        else:
            raise Exception("Cannot create timelapse of type " + type)


class Timelapse(object):

    def __init__(self):
        self.images = list()
        self.tmp_dir = "/tmp/"

    def addWMSImage(self, url):
        self.images.append(url)

    def fetchImages(self):
        p = Pool(5)
        imageList = p.map(fetchImage, self.images)
        p.close()
        p.join()
        return imageList

def fetchImage(url):
    return Image.fromarray(imageio.imread(url))

class GifTimeLapse(Timelapse):

    def create(self, delay=1, loop=0):
        print(str(datetime.datetime.now()) + ": Fetching images")
        save_dir = os.path.join(self.tmp_dir, ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)))
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        save_dir = save_dir + "/pygif.gif"
        frames = self.fetchImages()
        print(str(datetime.datetime.now()) + ": Creating gif image with fps ")
        frames[0].save(os.path.join(save_dir), save_all=True, append_images=frames[1:], duration=delay, loop=loop, optimize=False)
        return save_dir